/*
 * @copyright		© Copyright 2006-2011, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.colorpicker
 *
 * @revision		$Revision: 1.1.2.2 $
 */

jQuery.plugin( 'ep.ui.colorpicker', {
	plugin:	['jQuery.tmpl', 'jQuery.metaparse', 'jQuery.ui.widget', 'jQuery.ui.draggable', 'ep.core', 'ep.dict', 'ep.color', 'ep.colorpicker', 'ep.fn.contextOrientation', 'ep.ui.input'],
	dict:	'ep.dict',
	tmpl:	'ep.ui.colorpicker'
}, function($){

/*
 * @dictionary		ep.dict
 *
 * @translation		{ColorCode}
 *					{Transparent}
 *					{Restore}
 *					{UserColors}
 *					{MatchingColors}
 */

	var html			= $('html'),

		epColor			= ep.color,

		combinedColors	= function( color ){
			var hsb = epColor.stringToHsb(color),
				hsb1 = {},
				hsb2 = {},
				hsb3 = {},
				hsb4 = {};

			// set hue
			hsb1.h = hsb.h<340 ? hsb.h+20 : (hsb.h+20)-360;
			hsb2.h = hsb1.h;
			hsb3.h = hsb.h<240 ? hsb.h+120 : (hsb.h+120)-360;
			hsb4.h = hsb.h<260 ? hsb.h+100 : (hsb.h+100)-360;

			// set saturation
			hsb1.s = hsb.s>50 ? hsb.s-40 : hsb.s+40;
			hsb2.s = hsb.s>90 ? hsb.s-10 : hsb.s+10;
			hsb3.s = hsb.s>35 ? hsb.s-25 : hsb.s+25;
			hsb4.s = hsb2.s;

			// set brightness
			hsb1.b = hsb.b>60 ? hsb.b-40 : hsb.b+40;
			hsb2.b = hsb.b>80 ? hsb.b-20 : hsb.b+20;
			hsb3.b = 100;
			hsb4.b = hsb2.b;

			return {
					baseColor0 : '#'+epColor.stringToHex(color),
					baseColor1 : '#'+epColor.hsbToHex(hsb1),
					baseColor2 : '#'+epColor.hsbToHex(hsb2),
					baseColor3 : '#'+epColor.hsbToHex(hsb3),
					baseColor4 : '#'+epColor.hsbToHex(hsb4)
				};
		};

	$.widget( 'ep.ui.Colorpicker', {

		options: {
		//	extended:			{},
			autoOpen:			false,
			orientation:		'right',
			orientationAdjust:	[0,0],
			color:				'#fff',
			autoApply:			false,
			callback:			$.noop,
			transparent:		true
		},

		_create: function(){
			var self = this,
				o = self.options;

			self.elem = self.element
				.on( 'click', self.Proxy('open') )
				.sub(ep);

			self.activeCallback = $.noop;

			self.originalColor = o.color;
		},

		_init: function(){
			var self = this,
				o = self.options;

			if( o.autoOpen ){
				self.open();
			}

			if( o.extended && typeof o.extended.baseColors == 'string' ){
				o.extended.baseColors = combinedColors( o.extended.baseColors );
			}

			if( self.dialog ){
				if( o.autoApply ){
					self.applyButton.hide();
				}

				self.transparentInput
					.attr("disabled", !o.transparent)
					.parent()
					.css("visibility", o.transparent ? "visible" : "hidden");

				self._colorSelect( o.color, true );
			}
		},

		_build: function( remove ){
			if( !this.dialog ){
				var self = this,
					o = self.options,
					colorpicker;

				// load template and push named tmpl elements to current instance
				$.extend( self, $.tmpl( 'ep.ui.colorpicker', [o.extended || {}] ).tmplItem('elements') );

				self.dialog
					.hide()
					.appendTo('body')
					.dictParse('ep.dict',true)
					.draggable({
						containment:	'window',
						handle:			'.ep-uiSimpledialog-head'
					})
					.find('.ep-js')
						.metaparse();

				colorpicker = self.colorpicker = new ep.Colorpicker({
						margin:		10,
						size:		210,
//#JSCOVERAGE_IF false
						callback:	function( hex ){
							self._colorSelect( hex, "colorpicker" );
						}
//#JSCOVERAGE_ENDIF
					});

				self.pickerBox
					.on( 'mousedown', function(event){
						if( $(event.target).is('canvas.ep-colorpicker') ){
							self.transparentInput
								.attr('checked',false);
						}
					})
					.append(colorpicker.elem);

				self.closeButton
					.on('click', self.Proxy('cancel') );

				self.active = self.previewBox
					.on('click',function(){
						self._setActive();
					});

				self.hexInput
					.on( 'changeValue', function(){
						self._colorSelect( self.hexInput.val(), "hexInput" );
					});

				self.transparentInput
					.on('change',function(){
						self._colorSelect( self.transparentInput.is(":checked") ? "transparent" : "#ffffff", "transparentInput" );
					});

				self.applyButton
					.on('click',function(){
						self._colorApply();
						self.close();
					});

				if( self.userColors ){
					self.userColors
						.children('li')
						.on('click',function(){
							var elem = $(this),
								input =  elem.children('input');

							self._setActive( elem, function( color ){
								elem.css( 'background-color', color );
								input.val( color );
							});

							self._colorSelect( input.val(), true );
						})
						.on('dblclick',function(){
							self.previewBox.triggerHandler('click');
						});
				}

				if( self.baseColors ){
					self.baseColors
						.children('li')
						.on('click',function(){
							var elem = $(this),
								input =  elem.children('input'),
								val = input.val();

							self._colorSelect( val );
						});
				}

				if( self.originalBox ){
					self.originalBox
						.on( 'click', function(){
							var elem = $(this),
								color = '#' + ep.color.stringToHex( elem.css('background-color') );

							self._colorSelect( color );
						});
				}

				// set default active state
				self.previewBox.triggerHandler('click');

				self._init();
			}
		},

		_setOption: function( name, value ){
			this.Inherited(arguments);
			this._init();
		},

		_setActive: function( elem, callback ){
			elem = elem || this.previewBox;
			if( elem ){
				this.active.removeClass('active');
				this.active = elem.addClass('active');
				this.activeCallback = callback || $.noop;
			}
		},

		_colorSelect: function( color, context ){
			var self			= this,
				o				= self.options,
				isTransparent	= (color = color || o.color) == "transparent",
				hex				= isTransparent ? "ffffff" : epColor.stringToHex( color ),
				name			= isTransparent ? "transparent" : epColor.stringToName( color ),
				string,
				restore			= context === true; // no context but set also restorable color in original box

			// check whether transparent is allowed
			if( isTransparent && !o.transparent ){
				hex = "ffffff";
				name = "white";
				context = "";
			}

			if( hex || name ){
				hex = "#" + hex;
				string = name || hex;

				if( context !== "hexInput" ){
					self.hexInput.val( string );
				}
				if( context !== "transparentInput" ){
					self.transparentInput.attr( "checked", name == "transparent" );
				}
				if( context !== "colorpicker" ){
					self.colorpicker.setHex( hex || "ffffff", true );
				}

				o.color = string;

				self.elem.trigger('colorSelect',[string]);

				if( self.previewBox ){
					self.previewBox.css('background',hex);
				}

				if( restore && self.originalBox ){
					self.originalBox.css( 'background-color', string );
				}

				self.activeCallback( string );

				if( o.autoApply ){
					self._colorApply(string);
				}
			}
		},

		_colorApply: function(){
			var self = this,
				color = self.options.color;

			self.originalColor = color;
			self.options.callback(color);
			self.elem.trigger('colorApply',[color]);
		},

		colorSelect: function( color, original ){
			this._build();
			this._setActive();
			this._colorSelect( color, !!original );
		},

		colorApply: function(){
			this._build();
			this._setActive();
			this._colorSelect( undefined, true );
			this._colorApply();
		},

		colorReset: function(){
			this._build();
			this.elem.trigger('colorReset',[this.originalColor]);
			this._colorSelect( this.originalColor );
		},

		open: function(){
			var self = this,
				o = self.options;

			// close other colorpicker
			$('.ep-uiColorpicker-dialog').hide();

			self._build();
			self.dialog
				.show()
				.contextOrientation( self.elem, o.orientation, o.orientationAdjust );

			// move in other thread, required if colortool is created by click an element
			setTimeout(function(){
				html.on( 'click.uiColorpicker', self.Proxy('_close') );
			},1);

			this.elem.trigger('open');
		},

		cancel: function(){
			this.elem.trigger('cancel');
			this.close();
		},

		close: function(){
			var self		= this,
				extended	= self.options.extended;

			if( this.dialog ){
				this.dialog.hide();
				if( extended && extended.userColorsSave ){
					extended.userColorsSave( this.userColorsForm.serializeArray() );
				}
				html.off( 'click.uiColorpicker', self.Proxy('_close') );
				this.elem.trigger('close');
			}
		},

		_close: function( event ){
			var self	= this,
				elem	= self.element,
				target	= $(event.originalTarget);

			if( !jQuery.contains( elem[0], target[0] ) && target[0]!=elem[0] && !target.is('.ep-uiColorpicker-dialog, .ep-uiColorpicker-dialog *') ){
				this.close();
			}
		},

		destroy: function(){
			if( this.dialog ){
				this.dialog.remove();
			}
			this.Inherited(arguments);
		}

	});

});